if (typeof(simplemail) == "undefined" ) {simplemail = {};};

simplemail.SimpleMailNewMessage = function() {
  var pub           = {};
  pub.messageSent   = null,
  pub.tmpId         = null,
  pub.tabContainer  = null,
  pub.browserWindow = simplemail.SimpleMailUtils.getBrowserWindow(),
  pub.initialized   = false, // OnLoad event fires twice when iframe is present (FF bug)
  
  pub.initialize = function() {
    if (simplemail.SimpleMailNewMessage.initialized) 
      return;
    
    simplemail.SimpleMailNewMessage.initialized = true;
    simplemail.SimpleMailOverlay = simplemail.SimpleMailNewMessage.browserWindow.simplemail.SimpleMailOverlay;
    simplemail.SimpleMailNewMessage.tabContainer = simplemail.SimpleMailNewMessage.browserWindow.getBrowser().tabContainer;
    simplemail.SimpleMailNewMessage.initToolbarMode();
    simplemail.SimpleMailNewMessage.updateLayout();
    simplemail.SimpleMailNewMessage.populateCharset();
    simplemail.SimpleMailNewMessage.populateFontName();

    if (simplemail.SimpleMailPrefs.getBool("old")) {
      simplemail.SimpleMailDom.remove("nuvola-style");
      simplemail.SimpleMailDom.remove("oxygen-style");
      simplemail.SimpleMailDom.remove("australis-style");
    } else if (simplemail.SimpleMailPrefs.getBool("nuvola")) {
      simplemail.SimpleMailDom.remove("old-style");
      simplemail.SimpleMailDom.remove("oxygen-style");
      simplemail.SimpleMailDom.remove("australis-style");
    } else if (simplemail.SimpleMailPrefs.getBool("oxygen")) {
      simplemail.SimpleMailDom.remove("old-style");
      simplemail.SimpleMailDom.remove("nuvola-style");
      simplemail.SimpleMailDom.remove("australis-style");
    } else if (simplemail.SimpleMailPrefs.getBool("australis")) {
      simplemail.SimpleMailDom.remove("old-style");
      simplemail.SimpleMailDom.remove("nuvola-style");
      simplemail.SimpleMailDom.remove("oxygen-style");
    }

    if (simplemail.SimpleMailPrefs.getBool("newMessagesInWindow") || simplemail.SimpleMailPrefs.getBool("messagesInWindow")) { 
      simplemail.SimpleMailNewMessage.tmpId = simplemail.SimpleMailPrefs.getString('tmpId');
      var message = simplemail.SimpleMailPrefs.getObject('message');
      simplemail.SimpleMailOverlay.tmp[simplemail.SimpleMailNewMessage.tmpId] = message;
    }

    // Important: Do not allow mail messages to execute javascript,
    // Because this javascript would be executed with chrome privileges
    simplemail.SimpleMailDom.get("body").docShell.allowJavascript = false;

    simplemail.SimpleMailNewMessage.populateFrom();

    var doc = simplemail.SimpleMailDom.get("body").contentDocument;

    doc.addEventListener("keyup", simplemail.SimpleMailNewMessage.onKeyUp, false);
    doc.addEventListener("keypress", simplemail.SimpleMailNewMessage.onKeyPress, false);
    doc.addEventListener("mouseup", simplemail.SimpleMailNewMessage.onMouseUp, false);
    doc.addEventListener("drop", simplemail.SimpleMailNewMessage.dodrop, false);
    if (!window.simplemail.SimpleMailNewMessage.tmpId) simplemail.SimpleMailNewMessage.tmpId = /\?(.*)/.exec(location.href)[1];

    simplemail.SimpleMailNewMessage.setMessage(simplemail.SimpleMailOverlay.tmp[simplemail.SimpleMailNewMessage.tmpId]);
    simplemail.SimpleMailNewMessage.getMessage().copyTo(simplemail.SimpleMailOverlay.tmp[simplemail.SimpleMailNewMessage.tmpId]);

    doc.designMode = "on";
    doc.documentElement.contentEditable = "true";
    doc.body.spellcheck = "true";
    // Support for RTL language
    var lang = window.navigator.language
    if (lang == "ar" || lang == "he")
      doc.dir = "rtl";

    if (simplemail.SimpleMailPrefs.getBool("plaintextOnSend")) {
      simplemail.SimpleMailDom.get("smile").hidden = true;
      simplemail.SimpleMailDom.get("message-edit-toolbar").collapsed = true;
      if (!simplemail.SimpleMailPrefs.getBool("warningPlainText")) {
        simplemail.SimpleMailDom.get("headers-plaintextonsend").hidden = false;
      } else {
        simplemail.SimpleMailDom.get("headers-plaintextonsend").hidden = true;
      }
      simplemail.SimpleMailNewMessage.execCommand('FontName', 'Consolas');
      simplemail.SimpleMailNewMessage.execCommand('FontSize', '2');
      simplemail.SimpleMailNewMessage.execCommand('ForeColor', '#000000');
      doc.body.style.fontFamily = "Consolas";
      doc.body.style.fontSize = "0.8em";       
    } else {
      simplemail.SimpleMailDom.get("headers-plaintextonsend").hidden = true;
      var font = simplemail.SimpleMailPrefs.getObject("font");
      if (font["FontName"] == "Consolas") {
        simplemail.SimpleMailNewMessage.execCommand('FontName', 'Arial');
        simplemail.SimpleMailNewMessage.execCommand('FontSize', '2');
      }
    }

    simplemail.SimpleMailNewMessage.getFont();

    // Need to do this on timeout, otherwise iframe.contentWindow isn't focused
    setTimeout(function() {
      !simplemail.SimpleMailDom.get("to").value ? simplemail.SimpleMailDom.get("to").focus() :
        !simplemail.SimpleMailDom.get("subject").value ? simplemail.SimpleMailDom.get("subject").focus() 
        : simplemail.SimpleMailDom.get("body").contentWindow.focus();
    }, 0);
  
    if (!simplemail.SimpleMailPrefs.getBool("newMessagesInWindow")) { 
      simplemail.SimpleMailNewMessage.tabContainer.addEventListener("TabClose", simplemail.SimpleMailNewMessage.onTabClose, false);
    }
  },
  
  pub.onUnload = function () {
    // Need to remove event listener when message window is closed/reloaded
    if (simplemail.SimpleMailPrefs.getBool("newMessagesInWindow")) {
      simplemail.SimpleMailNewMessage.onClose();    
    } else {
      simplemail.SimpleMailNewMessage.tabContainer.removeEventListener("TabClose", simplemail.SimpleMailNewMessage.onTabClose, false);
    }
  },
  
  pub.onTabClose = function(event) {
    if (event.target.linkedBrowser.contentDocument == document) simplemail.SimpleMailNewMessage.onClose();
  },
  
  pub.onClose = function() {
    simplemail.SimpleMailNewMessage.saveDraft();
    simplemail.SimpleMailOverlay.tmp[simplemail.SimpleMailNewMessage.tmpId] = null; // Clean up temporary message
    simplemail.SimpleMailPrefs.clear('originalId');

    if (simplemail.SimpleMailPrefs.getBool("newMessagesInWindow") || simplemail.SimpleMailPrefs.getBool("messagesInWindow")) {
      simplemail.SimpleMailPrefs.clear('tmpId'); // Clean up temporary prefs
      simplemail.SimpleMailPrefs.clear('message');
      simplemail.SimpleMailPrefs.clear('originalId');
      simplemail.SimpleMailOverlay.refreshTab("chrome://simplemail/content/main/main.xul");
    }
  },
  
  pub.saveDraft = function(doSave, template) {
    var original = simplemail.SimpleMailOverlay.tmp[simplemail.SimpleMailNewMessage.tmpId];
    var message = simplemail.SimpleMailNewMessage.getMessage();
    if (message.equals(original)) return;
    
    if (!doSave) {
      doSave = !simplemail.SimpleMailNewMessage.messageSent &&
               !message.equals(original) && 
               simplemail.SimpleMailNewMessage.confirmSaveDraft();
    }
 
    if (doSave) {
      message.folderId = simplemail.SimpleMailModel.SimpleMailFolder.DRAFTS;
      message.template = template;
      simplemail.SimpleMailOverlay.addMessage(message);
      simplemail.SimpleMailOverlay.tmp[simplemail.SimpleMailNewMessage.tmpId] = simplemail.SimpleMailNewMessage.getMessage();
      if (simplemail.SimpleMailPrefs.getBool("newMessagesInWindow")) {
        var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
        var win = wm.getMostRecentWindow(null);
        simplemail.SimpleMailOverlay.refreshTab("chrome://simplemail/content/main/main.xul");
        setTimeout(function() {
          win.focus();
        }, 500);
      }
    }

    if (original == null) 
      return;

    if (original.id && (!original.template || template) && (simplemail.SimpleMailNewMessage.messageSent || doSave)) {
      setTimeout(function() { 
        simplemail.SimpleMailOverlay.dodeleteMessages([original]);
      }, 5000);
    }
  },
  
  pub.confirmSaveDraft = function() {
    var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
    var button = prompts.confirmEx(window, "Simple Mail",
                                   simplemail.SimpleMailBundle.getString("saveAsDraft"),
                                   prompts.STD_YES_NO_BUTTONS, null, null, null, null, {});
    return (button == 0); // "Yes"
  },
  
  pub.populateFrom = function() {
    var taccounts = simplemail.SimpleMailOverlay.storage.getAccounts();
    for (var i in taccounts) {
      if (!taccounts[i].smtp.host) 
        continue;
  
      var menuitem = document.createElement("menuitem");
      menuitem.setAttribute("label", taccounts[i]);
      menuitem.setAttribute("value", taccounts[i].id);
      simplemail.SimpleMailDom.get("from").menupopup.appendChild(menuitem);
    }
  },
  
  pub.filterAddressList = function(list, exclude) {
    var result = "";
    exclude = simplemail.SimpleMailModel.SimpleMailAddress.parse(exclude);
    var addresses = simplemail.SimpleMailModel.SimpleMailAddress.parseList(list);
    for(var i in addresses) {
      if (addresses[i].email != exclude.email) {
        result += result ? ", " + addresses[i] : addresses[i];
      }
    }
    return result;
  },
  
  pub.getMessage = function() {
    var message = new simplemail.SimpleMailModel.SimpleMailMessage();
    message.accountId = simplemail.SimpleMailDom.get("from").selectedItem.value;
    message.charset = simplemail.SimpleMailDom.get("charset").selectedItem.label;
    message.returnReceiptTo = simplemail.SimpleMailDom.get("returnReceipt").checked
                                  ? simplemail.SimpleMailDom.get("from").selectedItem.label : null;
    message.from = simplemail.SimpleMailDom.get("from").label;
    var account = simplemail.SimpleMailOverlay.storage.getAccount(message.accountId);
    message.replyTo = account.replyTo;
    message.to = simplemail.SimpleMailDom.get("to").value;
    message.cc = simplemail.SimpleMailDom.get("cc").value;
    message.bcc = simplemail.SimpleMailDom.get("bcc").value;
    message.subject = simplemail.SimpleMailDom.get("subject").value;
    message.html = simplemail.SimpleMailDom.get("body").contentDocument.body.innerHTML;
    return message;
  },
  
  pub.cleanAddresses = function(text) {
    var result = "";
    if (text.match(/(>,)/g)){
      text = text.replace (/(>,).*?/ig, ">;");
    }
    if (text.match(/(,)/g)){
      text = text.replace (/(,).*?/ig, ";");
    }    
    var list = ("" + text).split(/[;]/);
    for(var i in list){
      if (list[i].toString().match(/(@)/g)){
        result += result ? " " + list[i] + ", ": list[i] + ", ";
      } else {
        result += result ? " " + list[i] : list[i];
      }
    }
    result = result.replace(/(, )$/, "");
    return result;
   },
  
  pub.sendMail = function() {
    var message = simplemail.SimpleMailNewMessage.getMessage();
    if (!message.to && !message.cc && !message.bcc) {
      var confirmText = simplemail.SimpleMailBundle.getString("specifyRecipient");
      simplemail.SimpleMailUtils.alert(confirmText);
      return;
    }

    // If exist, delete coma in name
    simplemail.SimpleMailPrefs.setString('originalTo', message.to);
    simplemail.SimpleMailPrefs.setString('originalCc', message.cc);
    simplemail.SimpleMailPrefs.setString('originalBcc', message.bcc);
    
    message.to = simplemail.SimpleMailNewMessage.cleanAddresses(message.to);
    message.cc = simplemail.SimpleMailNewMessage.cleanAddresses(message.cc); 
    message.bcc = simplemail.SimpleMailNewMessage.cleanAddresses(message.bcc);
     
    simplemail.SimpleMailOverlay.sendMail(message, function() {
      simplemail.SimpleMailNewMessage.messageSent = true;
      var mainWindow = simplemail.SimpleMailOverlay.getMainWindow(true);
      window.close();
      if (mainWindow) mainWindow.history.go();
    });
  },

  // 1. Groups may be nested
  // 2. Have to work around circular references
  pub.expandGroups = function(string) {
    var addresses = simplemail.SimpleMailModel.SimpleMailAddress.parseList(string);
    var results   = [], processed = [], address;
    while ((address = addresses.shift())) {
      if (!address.email && !address.id)
        var address = simplemail.SimpleMailOverlay.storage.getAddressByName(address) || address;
        
      if (!address.group) {
        results.push(address);
      } else if (!processed[address.id]) {
        processed[address.id] = true;
        var group = simplemail.SimpleMailOverlay.storage.getAddressesByIds(address.group.split(",")); // userok
        addresses = addresses.concat(group);
      }
    }
    return results.join(", ");
  },

  pub.execCommand = function(command, param) {
    switch(command) {  
      case "ForeColor":
        param = simplemail.SimpleMailNewMessage.convertColor(param);
      case "FontName":
      case "FontSize":
        var font = simplemail.SimpleMailPrefs.getObject("font");
        font[command] = param;
        simplemail.SimpleMailPrefs.setObject("font", font);
    }
    simplemail.SimpleMailDom.get("body").contentDocument.execCommand(command, false, param);
    simplemail.SimpleMailDom.get("body").contentWindow.focus();
  },

  pub.insertLink = function() {
    var selection = simplemail.SimpleMailDom.get("body").contentWindow.getSelection();
    var range     = selection.getRangeAt(0);
  
    var url = { value: simplemail.SimpleMailText.getURL(selection) || "http://" };
  
    var parent = range.startContainer.parentNode;
    if (parent.nodeName == "A") {
      if (selection == "") range.selectNodeContents(parent);
      url.value = parent.href;
    }
  
    if (simplemail.SimpleMailUtils.prompt("enterLinkURL", url) && url.value) {
      if (selection != "") {
        simplemail.SimpleMailNewMessage.execCommand("Unlink");
        simplemail.SimpleMailNewMessage.execCommand("CreateLink", url.value);
      }
      else simplemail.SimpleMailNewMessage.execCommand("InsertHtml",
                       "<a href='" + url.value + "'>" + url.value + "</a>");
    }
    selection.collapseToEnd();
    simplemail.SimpleMailNewMessage.execCommand("Unlink");
  },
  
  pub.insertTable = function() {
    simplemail.SimpleMailNewMessage.execCommand("InsertHtml", "<table style='font: inherit;' border='1'>" +
                              "<tr><td></td><td></td></tr>" +
                              "<tr><td></td><td></td></tr></table>");
  },
  
  // Don't focus colorpicker when switching fields by Tab/Shift-Tab keys
  pub.skipColorpicker = function(event) {
    if (event.keyCode == event.DOM_VK_TAB) {
      if (!event.shiftKey) simplemail.SimpleMailDom.get("body").contentWindow.focus();
      else simplemail.SimpleMailDom.get("subject").select();
    }
  },

  pub.setMessage = function(message) {
    document.title = message.subject || " ";
    var body = simplemail.SimpleMailDom.get("body");
    simplemail.SimpleMailDom.get("returnReceipt").checked = message.returnReceiptTo ||
                                         simplemail.SimpleMailPrefs.getBool("returnReceipt");
    simplemail.SimpleMailUtils.selectItem("from", message.accountId);
    var me = simplemail.SimpleMailDom.get("from").selectedItem.label;
    simplemail.SimpleMailDom.get("to").value = simplemail.SimpleMailNewMessage.filterAddressList(simplemail.SimpleMailNewMessage.expandGroups(message.to), me);
    simplemail.SimpleMailDom.get("cc").value = simplemail.SimpleMailNewMessage.filterAddressList(simplemail.SimpleMailNewMessage.expandGroups(message.cc), me);
    simplemail.SimpleMailDom.get("bcc").value = simplemail.SimpleMailNewMessage.expandGroups(message.bcc);
    simplemail.SimpleMailUtils.selectItem("charset", message.charset ||
                                  simplemail.SimpleMailPrefs.getString("charset"), true);
    simplemail.SimpleMailDom.get("subject").value = message.subject || "";
    if (simplemail.SimpleMailPrefs.getBool("plaintextOnSend")) {
      if (message.html == undefined){
        var fragment = simplemail.SimpleMailUtils.parseHTML(body.contentDocument, "\r\n", true);
        body.contentDocument.body.innerHTML = "";
        body.contentDocument.body.appendChild(fragment);
      } else {
        var fragment = simplemail.SimpleMailUtils.parseHTML(body.contentDocument, message.html, true);
        body.contentDocument.body.innerHTML = "";
        body.contentDocument.body.appendChild(fragment);
      }      
    }
    else {
      if (message.html == undefined){
        var fragment = simplemail.SimpleMailUtils.parseHTML(body.contentDocument, "<br>", true);
        body.contentDocument.body.innerHTML = "";
        body.contentDocument.body.appendChild(fragment);
      } else {
        var fragment = simplemail.SimpleMailUtils.parseHTML(body.contentDocument, message.html, true);
        body.contentDocument.body.innerHTML = "";
        body.contentDocument.body.appendChild(fragment);
      }
    }

    simplemail.SimpleMailNewMessage.populateSubject();
    if (message.folderId !== 3) {
      simplemail.SimpleMailNewMessage.populateSignature();
    }
  },

  pub.populateSubject = function() {
    var accountId = simplemail.SimpleMailDom.get("from").selectedItem.value;
    var account = simplemail.SimpleMailOverlay.storage.getAccount(accountId);
    var lastSubject;
    var subject = account.subject || "";
    var regexp = new RegExp("^(" + (lastSubject || subject) + ")?$");
    simplemail.SimpleMailDom.get("subject").value = simplemail.SimpleMailDom.get("subject").value.replace(regexp, lastSubject = subject);
  },
  
  pub.getCurrentSignature = function() {
    var accountId = simplemail.SimpleMailDom.get("from").selectedItem.value;
    var account = simplemail.SimpleMailOverlay.storage.getAccount(accountId);
    var signature = account.signature.replace(/\r?\n/g, "<br>");

    if (signature) {
      var doc = simplemail.SimpleMailDom.get("body").contentDocument;
      var tmp = doc.createElement("div");
      var fragment = simplemail.SimpleMailUtils.parseHTML(doc, signature, true);
      tmp.appendChild(fragment);
      signature = tmp.innerHTML + "<br>";
    }
    return signature;
  },
  
  pub.populateSignature = function() {
    var body      = simplemail.SimpleMailDom.get("body");
    var signature = simplemail.SimpleMailNewMessage.getCurrentSignature();
    var prepend   = simplemail.SimpleMailDom.get("subject").value.match(/^(Re:|Fwd:)/i);
    var taccounts = simplemail.SimpleMailOverlay.storage.getAccounts();

    for (var i in taccounts) {
      if (lastSignature)
        var currentSignature = lastSignature
      else
        var currentSignature = taccounts[i].signature.replace(/\r?\n/g, "<br>");

      var regexp = new RegExp("(" + currentSignature + ")?$");

      if (body.contentDocument.body.innerHTML.match(regexp)){
        var signatureFragment = simplemail.SimpleMailUtils.parseHTML(body.contentDocument, signature, true);
        
        if (prepend) {
          if (lastSignature) 
            return;

          body.contentDocument.body.insertBefore(signatureFragment, body.contentDocument.body.childNodes[0]);
        } else {
          if (lastSignature)
            return;

          if (body.contentDocument.getElementById("Signature")){
            var node = body.contentDocument.getElementById("Signature");
            if (node.parentNode)
              node.parentNode.removeChild(node);
          }
          var tmp = body.contentDocument.createElement("div");
          tmp.setAttribute("id", "Signature");
          var tmp2 = body.contentDocument.createElement("div");
          tmp2.setAttribute("id", "Text");
          tmp.appendChild(tmp2)
          tmp2.appendChild(signatureFragment);
          body.contentDocument.body.appendChild(tmp)
        }
        
        var lastSignature = signature; 
        return;
      }
    }    
  },
  
  pub.updateLayout = function() {
    var hide = !simplemail.SimpleMailPrefs.getBool("fullEdit");
    simplemail.SimpleMailDom.get("cc-row").hidden = hide;
    simplemail.SimpleMailDom.get("bcc-row").hidden = hide;
    simplemail.SimpleMailDom.get("message-edit-toolbar").hidden = hide;
  },
  
  pub.switchLayout = function() {
    simplemail.SimpleMailPrefs.setBool("fullEdit", !simplemail.SimpleMailPrefs.getBool("fullEdit"));
    simplemail.SimpleMailNewMessage.updateLayout();
  },
  
  pub.isNewLine = function() {
    var selection = simplemail.SimpleMailDom.get("body").contentWindow.getSelection();
    var element = selection.focusNode;
  
    return selection.focusOffset == 0 ||
           element.nodeType != element.TEXT_NODE;
  },
  
  pub.insertQuoteBreak = function(event) {
    if (!simplemail.SimpleMailNewMessage.isNewLine()) return;
  
    var selection = simplemail.SimpleMailDom.get("body").contentWindow.getSelection();
  
    var blockquoteCount = 0;
    for(var element = selection.focusNode;
        element;
        element = element.parentNode)
    {
      if (element.nodeName == "BLOCKQUOTE") blockquoteCount++;
    }
  
    if (blockquoteCount) {
      for(var i = 0; i < blockquoteCount; i++) {
        simplemail.SimpleMailNewMessage.execCommand("Outdent");
      }
      event.preventDefault();
    }
  },
  
  pub.initToolbarMode = function() {
    var mode = simplemail.SimpleMailPrefs.getString("toolbarMode");
    simplemail.SimpleMailDom.get("message-toolbar").setAttribute("mode", mode);
  
    var menuitems = simplemail.SimpleMailDom.get("toolbar-menu").childNodes;
    var len = menuitems.length;
    for(var i = 0; i < len; i++) {
      if (menuitems[i].value == mode) {
        menuitems[i].setAttribute("checked", true);
      }
    }
  },
  
  pub.updateToolbarMode = function() {
    var menuitems = simplemail.SimpleMailDom.get("toolbar-menu").childNodes;
    var len = menuitems.length;
    for(var i = 0; i < len; i++) {
      if (menuitems[i].getAttribute("checked")) {
        simplemail.SimpleMailPrefs.setString("toolbarMode", menuitems[i].value);
        simplemail.SimpleMailDom.get("message-toolbar").setAttribute("mode", menuitems[i].value);
      }
    }
  },
  
  pub.populateFontName = function() {
    var popup = simplemail.SimpleMailDom.get("fontName").menupopup;
    var fonts = simplemail.SimpleMailUtils.getFontList();
    var len = fonts.length;
    for(var i = 0; i < len; i++) {
      var menuitem = document.createElement("menuitem");
      menuitem.setAttribute("label", fonts[i]);
      menuitem.setAttribute("value", fonts[i]);
      popup.appendChild(menuitem);
    }
  },
  
  pub.populateCharset = function() {
    var popup = simplemail.SimpleMailDom.get("charset").menupopup;
    var charsets = simplemail.SimpleMailUtils.getDefaultCharsets();
    var len = charsets.length;
    for(var i = 0; i < len; i++) {
      var menuitem = document.createElement("menuitem");
      menuitem.setAttribute("label", charsets[i]);
      popup.appendChild(menuitem);
    }
  },
  
  pub.onKeyUp = function(event) {
    if (!simplemail.SimpleMailPrefs.getBool("plaintextOnSend")) {
      simplemail.SimpleMailNewMessage.updateLink(event);
    }
    simplemail.SimpleMailNewMessage.getFont();
  },
  
  pub.onKeyPress = function(event) {
    if (event.keyCode == event.DOM_VK_RETURN) {
      if (event.ctrlKey) {
        simplemail.SimpleMailNewMessage.sendMail();
      }
      else simplemail.SimpleMailNewMessage.insertQuoteBreak(event);
    }
    else {
      var char = String.fromCharCode(event.charCode);
      switch(char) {
        case " ":
        case ",": simplemail.SimpleMailNewMessage.execCommand("Unlink");
                  break;
      }
    }
  },
  
  pub.onMouseUp = function(event) {
    if (event.ctrlKey) {
      var font = simplemail.SimpleMailPrefs.getObject("font");
      simplemail.SimpleMailNewMessage.execCommand("ForeColor", font.ForeColor);
    }
    if (!simplemail.SimpleMailPrefs.getBool("plaintextOnSend")) {
      simplemail.SimpleMailNewMessage.updateLink(event);
    }
    simplemail.SimpleMailNewMessage.getFont();
  },
  
  pub.convertColor = function(color) {
    var match = color.match(/rgb\((\d+), (\d+), (\d+)\)/);
    
      function toHex(value) {
        value = parseInt(value).toString(16).toUpperCase();
        return value.length == 1 ? "0" + value : value;
      }
    
    if (match) {

      color = "#" + toHex(match[1]) + toHex(match[2]) + toHex(match[3]);
    }
    if (color == "#000000") color = "normal";
    return color;
  },
  
  pub.disableCss = function(disable) {
    var doc = simplemail.SimpleMailDom.get("body").contentDocument;
    var len = doc.styleSheets.length;
    for(var i = 0; i < len; i++) {
      doc.styleSheets[i].disabled = disable;
    }
  },
  
  pub.setFont = function() {
    var selection = simplemail.SimpleMailDom.get("body").contentWindow.getSelection();
    var range = selection.getRangeAt(0);
  
    if (selection == "" &&
        range.startContainer.nodeName == "BODY") // This is a new line
    {
      var font = simplemail.SimpleMailPrefs.getObject("font");
      for(var i in font) {
        simplemail.SimpleMailNewMessage.execCommand(i, font[i]);
      }
    }
  },

  pub.getFont = function() {
    simplemail.SimpleMailNewMessage.setFont();
    simplemail.SimpleMailNewMessage.disableCss(true);
    var doc = simplemail.SimpleMailDom.get("body").contentDocument;
    simplemail.SimpleMailUtils.selectItem("fontName", doc.queryCommandValue("FontName"), true);
    simplemail.SimpleMailUtils.selectItem("fontSize", doc.queryCommandValue("FontSize"));
    simplemail.SimpleMailDom.get("color").color = doc.queryCommandValue("ForeColor") || "#000000";
    simplemail.SimpleMailNewMessage.getTextProperties();
    simplemail.SimpleMailNewMessage.getTextJustify();
    simplemail.SimpleMailNewMessage.disableCss(false);
  },

  pub.getTextProperties = function() {
    var selection = simplemail.SimpleMailDom.get("body").contentWindow.getSelection();
  
    var bold, italic, underline, bullet, number;
    for(var element = selection.focusNode;
        element;
        element = element.parentNode)
    {
      bold = bold ||
             element.nodeName == "B" ||
             element.nodeName == "STRONG" ||
             element.style && element.style.fontWeight == "bold";
  
      italic = italic ||
               element.nodeName == "I" ||
               element.nodeName == "EM" ||
               element.style && element.style.fontStyle == "italic";
  
      underline = underline ||
                  element.nodeName == "U" ||
                  element.style && element.style.textDecoration == "underline";
  
      bullet = bullet || element.nodeName == "UL";
      number = number || element.nodeName == "OL";
    }
    simplemail.SimpleMailDom.get("bold").checked = bold;
    simplemail.SimpleMailDom.get("italic").checked = italic;
    simplemail.SimpleMailDom.get("underline").checked = underline;
    simplemail.SimpleMailDom.get("bullet").checked = bullet;
    simplemail.SimpleMailDom.get("number").checked = number;
  },
  
  pub.getTextJustify = function() {
    var doc = simplemail.SimpleMailDom.get("body").contentDocument;
    var justify = doc.queryCommandValue("justifyLeft");
  
    var names = ["left", "center", "right"];
    for(var i in names) {
      simplemail.SimpleMailDom.get(names[i]).checked = names[i] == justify;
    }
  },
  
  pub.updateLink = function(event) {
    var doc = simplemail.SimpleMailDom.get("body").contentDocument;
    var selection = simplemail.SimpleMailDom.get("body").contentWindow.getSelection();
    var range = selection.getRangeAt(0); 
  
    var node = range.startContainer;
    var parent = node.parentNode;
    if (parent && parent.nodeName == "A") {
      var url = simplemail.SimpleMailText.getURL(node.textContent);
      if (url && url != parent.href) parent.href = url;
    }
    else {
      var url;
      while(node && node.nodeType == node.TEXT_NODE &&
            (url = simplemail.SimpleMailText.getURL(node.textContent)))
      {
        var start = node.textContent.indexOf(url);
        range = selection.getRangeAt(0);
        range.selectNode(node);
        range.setStart(node, start);
        range.setEnd(node, start + url.length);
        simplemail.SimpleMailNewMessage.execCommand("CreateLink", url);
  
        range = selection.getRangeAt(0);
        range.collapse(false);
        var link = range.startContainer.parentNode;
        node = link.nextSibling;
      }
    }
  },
  
  pub.populateSmiles = function() { 
    var menupopup = simplemail.SimpleMailDom.get("smile").firstChild;
    simplemail.SimpleMailUtils.clearElement(menupopup);
    simplemail.SimpleMailNewMessage.populateSmilesFrom(simplemail.SimpleMailFile.getSmilesDirectory(), "chrome://simplemail/skin/smiles/");
    simplemail.SimpleMailNewMessage.populateSmilesFrom(simplemail.SimpleMailFile.getDirectory("smiles"), "simplemail://smiles/");
    var horizontalBox = document.createElement("hbox");
    menupopup.appendChild(horizontalBox);
  
    var browse = document.createElement("label");
    browse.setAttribute("value", "...");
    browse.addEventListener("click", simplemail.SimpleMailNewMessage.browseSmiles, false);
    horizontalBox.appendChild(browse);
  },
  
  pub.populateSmilesFrom = function(directory, baseUrl) {
    var SMILES_PER_ROW = 5;
    var horizontalBox, i = 0;
    var menupopup = simplemail.SimpleMailDom.get("smile").firstChild;
    
    simplemail.SimpleMailFile.forEachFile(directory, function(file) {
      var image = document.createElement("image");
      image.setAttribute("src", baseUrl + file.leafName);
  
      var verticalBox = document.createElement("vbox");
      verticalBox.appendChild(image);

      if (i++ % SMILES_PER_ROW == 0) {
        horizontalBox = document.createElement("hbox");
        horizontalBox.setAttribute("align", "baseline");
        menupopup.appendChild(horizontalBox);
      }
      horizontalBox.appendChild(verticalBox);
    });
  },
  
  pub.browseSmiles = function() {
    simplemail.SimpleMailDom.get("smile").firstChild.hidePopup();
  
    var files = simplemail.SimpleMailFile.chooseFile("multiple", ["images"]);
    var targetDir = simplemail.SimpleMailFile.getDirectory("smiles");
  
    for(var i in files) {
      var target = targetDir.clone();
      target.append(files[i].leafName);
      if (target.exists()) target.remove(false);
      files[i].copyTo(targetDir, files[i].leafName);
    }
    simplemail.SimpleMailDom.get("smile").firstChild.showPopup();
  },
  
  pub.insertSmile = function(element) {
    var url = element.getAttribute("src") || element.getAttribute("image");
    if (url) {
      simplemail.SimpleMailNewMessage.execCommand("InsertImage", url);
      simplemail.SimpleMailDom.get("smile").firstChild.hidePopup();
    }
  },
  
  pub.removeSmile = function(element) {
    var url = element.getAttribute("src");
    var name = simplemail.SimpleMailFile.getFileName(url);
    var file = simplemail.SimpleMailFile.getDirectory("smiles");
    file.append(name);
    try {
      file.remove(false);
      simplemail.SimpleMailNewMessage.populateSmiles();
    }
    catch(e) {}
  },
  
  pub.onSmileClick = function(event) {
    if (event.button == 0) simplemail.SimpleMailNewMessage.insertSmile(event.target);
    if (event.button == 2) simplemail.SimpleMailNewMessage.removeSmile(event.target);
  },
  
  pub.insertImage = function() {
    var files = simplemail.SimpleMailFile.chooseFile("multiple", ["images"]);
    if (!files) return;
    
    for(var i in files) {
      var url = simplemail.SimpleMailFile.getFileURL(files[i]);
      simplemail.SimpleMailNewMessage.execCommand("InsertImage", url);
    }
  },  
  
  pub.attachFile = function() {
    var files = simplemail.SimpleMailFile.chooseFile("multiple");
    if (!files) return;
    var html = "";

    for(var i in files) {
      var url = simplemail.SimpleMailFile.getFileURL(files[i]);
      var filename = simplemail.SimpleMailFile.getFileName(url);
      var background = simplemail.SimpleMailMessageEncoder.getIconForAttachment(filename);
      filename = filename.toString().replace(/[\'] */gi, "_");
      html = "<a class='attachment' style='background: url(" + background + ")center left no-repeat;' href='" + url + "'>" + filename + "</a>";
      simplemail.SimpleMailNewMessage.pushAttachFile(html);
    } 
  },
  
  pub.pushAttachFile  = function(html){
    var body = simplemail.SimpleMailDom.get("body");
    var range = document.createRange();
    var documentFragment = range.createContextualFragment(html)
    body.contentDocument.body.appendChild(documentFragment);
    var message = simplemail.SimpleMailNewMessage.getMessage();

    // Need this to update RichText Editor
    simplemail.SimpleMailNewMessage.execCommand("selectAll");
    simplemail.SimpleMailNewMessage.execCommand("delete");
    simplemail.SimpleMailNewMessage.execCommand("InsertHtml", message.html);
  },  
  
  pub.dodrop = function(event) {
    var MOZ_URL = "text/x-moz-url";
    var html = ""; 
    var dt = event.dataTransfer;
    var count = dt.mozItemCount;
    var origin = dt.mozSourceNode;

    if (origin == null) {
      for (var i = 0; i < count; i++) {
        var files = dt.mozGetDataAt(MOZ_URL, i);
        var url = files;
        var ttype = simplemail.SimpleMailFile.getMimeType(url);
        if (ttype.match(/^image/i)){
          event.preventDefault();
          var filename = simplemail.SimpleMailFile.getFileName(url);
          var background = simplemail.SimpleMailMessageEncoder.getIconForAttachment(filename);
          html = "<a class='attachment' style='background: url(" + background + ")center left no-repeat;' href='" + url + "'>" + filename + "</a>";
          simplemail.SimpleMailNewMessage.pushAttachFile(html); 
        }  
        else if (url.match(/^file:/)) { 
          var filename = simplemail.SimpleMailFile.getFileName(url);
          var background = simplemail.SimpleMailMessageEncoder.getIconForAttachment(filename);
          html = "<a class='attachment' style='background: url(" + background + ")center left no-repeat;' href='" + url + "'>" + filename + "</a>";
          simplemail.SimpleMailNewMessage.pushAttachFile(html); 
        }
      }
    }
  }
  return pub;
}();

simplemail.AutoComplete = function () {
  var pub    = {};
  pub.values = [],
  pub.REGEXP = /\s*([^,;]*)$/,

  pub.onInput = function(textbox) {
    if (textbox.userAction == "typing") // SeaMonkey
      simplemail.AutoComplete.values[textbox.id] = textbox.value;

    var result = simplemail.AutoComplete.REGEXP.exec(textbox.value);
    textbox.setAttribute("autocompletesearchparam", result[1]);
  },

  pub.onSearchComplete = function(textbox) {
    simplemail.AutoComplete.values[textbox.id] = textbox.value;
  },

  pub.onTextEntered =  function(textbox) {
    var value = simplemail.AutoComplete.values[textbox.id] || "";
    if (value != textbox.value) {
      textbox.value = value.replace(simplemail.AutoComplete.REGEXP, simplemail.SimpleMailNewMessage.expandGroups(textbox.value));
      textbox.setAttribute("autocompletesearchparam", "");
    }
    return false;
  }
  
  return pub;
}();