var actionTree, actions;

function initActions(aActions) {
  actions = aActions;
  actionTree = simplemail.SimpleMailDom.get("action-tree");
  actionTree.view = actionTreeView;
  if (actionTree.view.rowCount > 0) 
    actionTree.view.selection.select(0);
}

var actionTreeView = {
  get rowCount() {
    return actions.length;
  },
  getCellText: function(row, column) {
    var name = column.id.replace(/action-/, "");
    var value = actions[row][name];

    if (name == "param") {
      var action = actions[row]["name"];
      switch(action) {
        case "color": return "██";
        case "folder":
            var folder = simplemail.SimpleMailOverlay.storage.getFolder(value) ||
                         simplemail.SimpleMailOverlay.storage.getFolder(simplemail.SimpleMailModel.SimpleMailFolder.INBOX);
            return folder.getName();
      }
    }
    return simplemail.SimpleMailBundle.getString(value) || value;
  },
  getCellProperties: function(row, column, props) {
    var name = column.id.replace(/action-/, "");
    var value = actions[row][name];
    if (name == "param" && actions[row]["name"] == "color") {
      if (props) {
        var atom = actionTreeView.atomService.getAtom(SimpleMailCOLORS[value]);
        props.AppendElement(atom);
      } else return SimpleMailCOLORS[value];
    }
  },
  action: editAction,
  delete: deleteActions
};

simplemail.DefaultTreeView.extend(actionTreeView);

function onSelectAction() {
  simplemail.SimpleMailDom.get("action-edit").disabled =
  simplemail.SimpleMailDom.get("action-delete").disabled = !actionTree.view.selection.getRangeCount();
}

function editAction(add) {
  var action = add ? simplemail.SimpleMailModel.SimpleMailAction.create() : actions[actionTree.currentIndex];
  var param = { action: action };

  openDialog("chrome://simplemail/content/preferences/filter/action.xul",
             "simplemail-preferences-filter-action-window",
             "chrome,centerscreen,modal,resizable", param);
  if (!param.action) return;

  if (add) {
    actions.push(action);
    actionTreeView.treebox.rowCountChanged(actionTreeView.rowCount - 1, 1);
    actionTree.view.selection.select(actionTreeView.rowCount - 1);
  } else {
    actionTreeView.treebox.invalidateRow(actionTree.currentIndex);
  }
}

function deleteActions() {
  if (simplemail.SimpleMailUtils.confirm(simplemail.SimpleMailBundle.getString("deleteAction"))) {
    actionTreeView.forEachSelected(function(index) {
      actions.splice(index, 1);
      actionTreeView.treebox.rowCountChanged(index, -1);
    });
  }
}
